class Xml {
	static Files := {Input: {}
			, Output: {}}
		, XmlStylesheets := {}
	
	Init()
	{
		this.Files.Input.Xml := ComObjCreate("MSXML2.DOMDocument.6.0")
		this.Files.Input.Xml.async := false
		this.Files.Input.Xml.validateOnParse := Config.ValidateOnParse
		this.Files.Input.Xml.setProperty("ProhibitDTD", Config.ProhibitDTD)
		this.Files.Input.Xml.resolveExternals := Config.ResolveExternals
		this.Files.Input.Xml.setProperty("UseInlineSchema", Config.UseInlineSchema)
	}
	
	InputFile[]
	{
		set {
			if value {
				this.Files.Input.Xml.load(value)
				if (this.Files.Input.Xml.parseError.errorCode) {
					MsgBox, 8208, % Txt.Get("XmlInputParseErrorMsgBoxTitle")
						, % Txt.Get("XmlInputParseErrorMsgBoxText"
							, {XmlInputFile: value
								, ParseErrorLine: this.Files.Input.Xml.parseError.line
								, ParseErrorSrcText: this.Files.Input.Xml.parseError.srcText
								, ParseErrorReason: this.Files.Input.Xml.parseError.reason})
					return 0
				}
				SplitPath, value,,,, FileNameNoExt
				this.Files.Input.FileNameNoExt := FileNameNoExt
				this.XmlStylesheets := []
				XmlStylesheetNodes := this.Files.Input.Xml.selectNodes("/processing-instruction('xml-stylesheet')")
				if XmlStylesheetNodes.Length() {
					Stylesheet := ComObjCreate("MSXML2.DOMDocument.6.0")
					Stylesheet.async := false 
					Stylesheet.validateOnParse := false
					for item in XmlStylesheetNodes {
						Stylesheet.loadXml("<stylesheet " item.text "/>")
						if (XslStylesheetNode := Stylesheet.selectSingleNode("/stylesheet[@href][@type='text/xsl']")) {
							Media := (StylesheetMedia := Stylesheet.selectSingleNode("/stylesheet/@media"))
								? StylesheetMedia.text
								: "all"
							StylesheetHref := Stylesheet.selectSingleNode("/stylesheet/@href")
							Href := StylesheetHref.text
							if FileExist(Href) {
								if not this.XmlStylesheets.HasKey(Media) {
									this.XmlStylesheets[Media] := []
								}
								this.XmlStylesheets[Media].Push(Href)
							} else {
								MsgBox, 8208, % Txt.Get("FileNotFoundMsgBoxTitle"), % Txt.Get("FileNotFoundMsgBoxText", {File: Href})
							}
						}
					}
					for key in this.XmlStylesheets {
						if MoreThanOne
							return "MediaChoice"
						MoreThanOne := 1
					}
					return "XmlStylesheets"
				}
				return 1
			}
		}
	}
	
	XmlStylesheetMediaChoiceString[]
	{
		get {
			for key in this.XmlStylesheets {
				if MoreThanOne
					String .= "|"
				String .= key
				MoreThanOne := 1
			}
			return String
		}
	}
	
	Transform()
	{
		XsltList := Config.XsltList["CompleteList"]
		AllowDocumentFunction := Config.AllowDocumentFunction
		AllowXsltScript := Config.AllowXsltScript
		ResolveExternals := Config.ResolveExternals
		ValidateOnParse := Config.ValidateOnParse
		ProhibitDTD := Config.ProhibitDTD
		UseInlineSchema := Config.UseInlineSchema
		
		for key, item in XsltList {
			XsltDoc%A_Index% := ComObjCreate("MSXML2.FreeThreadedDOMDocument.6.0")
			XsltDoc%A_Index%.async := false
			XsltDoc%A_Index%.validateOnParse := false
			XsltDoc%A_Index%.setProperty("AllowDocumentFunction", AllowDocumentFunction)
			XsltDoc%A_Index%.setProperty("AllowXsltScript", AllowXsltScript)
			XsltDoc%A_Index%.resolveExternals := ResolveExternals
			XsltDoc%A_Index%.setProperty("SelectionNamespaces", "xmlns:xsl='http://www.w3.org/1999/XSL/Transform'")
			XsltDoc%A_Index%.load(item.Path)
			if (XsltDoc%A_Index%.parseError.errorCode) {
				MsgBox, 8208, % Txt.Get("XsltParseErrorMsgBoxTitle")
					, % Txt.Get("XsltParseErrorMsgBoxText"
						, {XsltFile: item.Path
							, ParseErrorLine: XsltDoc%A_Index%.parseError.line
							, ParseErrorSrcText: XsltDoc%A_Index%.parseError.srcText
							, ParseErrorReason: XsltDoc%A_Index%.parseError.reason})
				return
			}
			Xslt%A_Index% := ComObjCreate("MSXML2.XSLTemplate.6.0")
			Xslt%A_Index%.stylesheet := XsltDoc%A_Index%
			XsltProc%A_Index% := Xslt%A_Index%.createProcessor()
			XsltProc%A_Index%.input := (A_Index = 1) ? this.Files.Input.Xml : TempXml
			for parameterName, parameterValue in item.Parameter {
				XsltProc%A_Index%.addParameter(parameterName, parameterValue)
			}
			XsltProc%A_Index%.transform()
			if (A_Index = XsltList.length()) {
				OutputMethodNode := XsltDoc%A_Index%.selectSingleNode("/xsl:stylesheet/xsl:output/@method")
				OutputMethod := (OutputMethodNode) ? OutputMethodNode.text : "xml"
				if OutputMethod not in xml,html,text
					OutputMethod := "xml"
				this.Files.Output.Method := OutputMethod
				this.Files.Output.Content := XsltProc%A_Index%.output
			} else {
				TempXml := ComObjCreate("MSXML2.DOMDocument.6.0")
				TempXml.async := false
				TempXml.validateOnParse := ValidateOnParse
				TempXml.setProperty("ProhibitDTD", ProhibitDTD)
				TempXml.resolveExternals := ResolveExternals
				TempXml.setProperty("UseInlineSchema", UseInlineSchema)
				TempXml.loadXml(XsltProc%A_Index%.output)
				if (TempXml.parseError.errorCode) {
					MsgBox, 8208, % Txt.Get("TempXmlParseErrorMsgBoxTitle")
						, % Txt.Get("TempXmlParseErrorMsgBoxText"
							, {XsltFile: item.Path
								, ParseErrorLine: TempXml.parseError.line
								, ParseErrorSrcText: TempXml.parseError.srcText
								, ParseErrorReason: TempXml.parseError.reason})
					return
				}
			}
			XsltProc%A_Index% := ""
			Xslt%A_Index% := ""
			XsltDoc%A_Index% := ""
		}
		return 1
	}
	
	InputFileNameNoExt[]
	{
		get {
			return this.Files.Input.FileNameNoExt
		}
	}
	
	OutputMethod[]
	{
		get {
			return this.Files.Output.Method
		}
	}
	
	WriteXmlResultFile(Path)
	{
		XmlResultFile := FileOpen(Path, "w", "UTF-8-RAW")
		if !IsObject(XmlResultFile)
		{
			MsgBox, 8208, % Txt.Get("XmlResultFileErrorMsgBoxTitle"), % Txt.Get("XmlResultFileErrorMsgBoxText", {ResultFile: Path})
			return
		}
		XmlResultFile.Write(this.Files.Output.Content)
		XmlResultFile.Close()
	}
}